#include "backend"

bool Backend::connect() {
    // Assume the backend is dead
    islive = false;
    
    // Create client socket
    if ( (clsocket = socket (PF_INET, SOCK_STREAM, 0)) < 0 )
	throw ((Error)"Failed to create client socket: " + strerror(errno));

    // Resolve hostname, prepare binding
    struct hostent *hostaddr;
    if (! (hostaddr = gethostbyname(bdef.server().c_str())) )
	throw ((Error)"Failed to resolve backend host '" + bdef.server());
    struct sockaddr_in backendaddr;
    backendaddr.sin_family = AF_INET;
    backendaddr.sin_port = htons(bdef.port());
    memcpy ((char *) &backendaddr.sin_addr.s_addr,
	    hostaddr->h_addr_list[0], hostaddr->h_length);

    // Client socket goes into nonblocking mode, so we can connect
    // and enforce a timeout later.
    int flags;
    if ( (flags = fcntl (clsocket, F_GETFL, 0)) == -1 )
	throw ((Error) "Failed to get fd flags: " + strerror(errno));
    if (fcntl (clsocket, F_SETFL, flags | O_NONBLOCK) == -1)
	throw ((Error) "Failed to fd in nonblocking mode: " + strerror(errno));

    // Do the connect
    int conres = ::connect (clsocket, (struct sockaddr *)&backendaddr,
			 sizeof(backendaddr));
    int conerrno = errno;

    // Put socket again in blocking mode.
    if (fcntl (clsocket, F_SETFL, flags) == -1)
	throw ((Error) "Failed to put fd in blocking mode: " +
	       strerror(errno));

    // Check on the outcome of the connect
    if (!conres || conerrno == EINPROGRESS) {
	// Wait for socket to go writable.
	Fdset fdset (config.backend_timeout());
	fdset.add (clsocket);
	if (fdset.readwriteable() == -1 && fdset.writeable() == clsocket)
	    islive = true;
    }

    /* 
    msg << "Back end " << bdef.server() << ":" << bdef.port();
    if (islive)
	msg << " is live on socket " << clsocket << "\n";
    else
	msg << " is NOT live\n";
    */
    
    return (islive);
}
