#include "config"
#include "../build/usage.h"

using namespace std;

void Config::parsecmdline (int ac, char **av) {
    // Prepare invoking command line.
    string cmdline;
    for (int i = 0; i < ac; i++) {
	if (i)
	    cmdline += ' ';
	cmdline += av[i];
    }
    // Not a single argument? Usage.
    if (ac == 1)
	throw ((Error) "Bad command line '" + cmdline + "'\n" + USAGE);

    static struct option longopts[] = {
	{ "allow-from",		 required_argument, 0, 'a' },
	{ "deny-from",		 required_argument, 0, 'A' },
	{ "backend", 	 	 required_argument, 0, 'b' },
	{ "buffer-size",  	 required_argument, 0, 'B' },
	{ "checkup-interval",  	 required_argument, 0, 'c' },
	{ "debug", 	 	 no_argument, 	    0, 'D' },
	{ "dispatch-mode", 	 required_argument, 0, 'd' },
	{ "foreground", 	 no_argument, 	    0, 'f' },
	{ "help", 		 no_argument, 	    0, 'h' },
	{ "add-server-header",   required_argument, 0, 'H' },
	{ "max-connections", 	 required_argument, 0, 'm' },
	{ "prefix-timestamp",	 no_argument,       0, 'P' },
	{ "pidfile",		 required_argument, 0, 'p' },
	{ "server", 		 required_argument, 0, 's' },
	{ "sticky-http", 	 no_argument, 	    0, 'S' },
	{ "backend-timeout", 	 required_argument, 0, 't' },
	{ "client-timeout", 	 required_argument, 0, 'T' },
	{ "verbose", 		 no_argument, 	    0, 'v' },
	{ "version", 		 no_argument, 	    0, 'V' },
	{ "wakeup-interval",	 required_argument, 0, 'w' },
	{ "add-xr-version", 	 no_argument, 	    0, 'X' },
	{ "add-x-forwarded-for", no_argument, 	    0, 'x' },
	{ 0, 			 0, 		    0, 0 }
    };
    
    int opt;
    bool backend_set = false;
    while ( (opt = getopt_long (ac, av, "?a:A:B:b:c:Dd:fhH:m:p:Ss:t:T:vVw:xX",
				longopts, 0)) > 0) {
	switch (opt) {
	case 'a':
	    addallow (optarg);
	    break;
	case 'A':
	    adddeny (optarg);
	    break;
	case 'b':
	    setbackend (optarg);
	    backend_set = true;
	    break;
	case 'B':
	    bufsize = (unsigned)setinteger (optarg);
	    break;
	case 'c':
	    checkup = setinteger (optarg);
	    break;
	case 'D':
	    verbose_flag = true;
	    debug_flag = true;
	    break;
	case 'd':
	    setdispatchmode (optarg);
	    break;
	case 'f':
	    foreground_mode = true;
	    break;
	case 'h':
	    throw ((Error) USAGE);
	    break;
	case 'H':
	    addserverheader (optarg);
	    break;
	case 'm':
	    max_conn = (unsigned)setinteger (optarg);
	    break;
	case 'P':
	    prefix_timestamp = true;
	    break;
	case 'p':
	    pid_file = optarg;
	    break;
	case 's':
	    setserver (optarg);
	    break;
	case 'S':
	    sticky_http = true;
	    break;
	case 't':
	    b_timeout = setinteger (optarg);
	    break;
	case 'T':
	    c_timeout = setinteger (optarg);
	    break;
	case 'v':
	    verbose_flag = true;
	    break;
	case 'V':
	    cout << "This is XR V"    << VER << "\n"
		 << "Written by: "    << AUTHOR << "\n"
		 << "Maintained by: " << MAINTAINER << "\n"
		 << "Please report bugs to the maintainer.\n";
	    exit (0);
	case 'w':
	    wakeup = setinteger (optarg);
	    break;
	case 'X':
	    add_xr_version = true;
	    break;
	case 'x':
	    add_x_forwarded_for = true;
	    break;
	default:
	    throw ((Error) "Unknown flag, try 'xr -h' for usage");
	    break;
	}
    }

    if (ac != optind)
	throw ((Error) "Bad command line '" + cmdline + "'\n" + USAGE);

    if (!backend_set)
	throw ((Error)
	       "No backend defined, use '-b...' at least once, "
	       "or try 'xr -h' for usage");

    msg ("+--------------------------------------------+\n");
    msg ("| Welcome to xr V" VER "                        |\n");
    msg ("| Copyright (c) Karel Kubat <karel@kubat.nl> |\n");
    msg ("| Distributed under GPLV3.                   |\n");
    msg ("+--------------------------------------------+\n");

    msg ("Invoking command line: '" + cmdline + "'\n");
}
