#include "httpdispatcher"

void HttpDispatcher::handle() {

    // The client request was already retrieved before starting the
    // dispatcher. We can continue by applying server-directed headers.
    clientrequest.setversion("1.0");
    clientrequest.setheader ("Connection", "close");
    clientrequest.setheader ("Proxy-Connection", "close");

    // Apply other server-side directed info
    if (config.addxrversion())
	clientrequest.setheader ("XR", VER);
    if (config.addxforwardedfor())
	clientrequest.addheader ("X-Forwarded-For", clientip());
    for (unsigned n = 0; n < config.nserverheaders(); n++)
	clientrequest.addheader (config.serverheader(n));

    // Send to server
    if (!sendclientrequest() ) {
	msg ("Failed to send client request to back end, not processing\n");
	return;
    }

    // Get server's response
    if (!getserverresponse()) {
	msg ("Failed to receive a valid back end response, not processing\n");
	return;
    }

    // Apply client-side directed info. Smuggle in stickiness if required.
    if (config.addxrversion())
	serverresponse.setheader ("XR", VER);
    if (config.stickyhttp() && !issticky()) {
	ostringstream o;
	o << targetbackend();
	serverresponse.setheader ("Set-Cookie", "XRTarget=" + o.str());
    }

    // Flush server buffer to the client
    writechunk (clientfd(), serverresponse.data(), serverresponse.size());
    
    // Then switch to the TCP copy/thru protocol.
    TcpDispatcher::handle();
}
