
#include "../sys/sys"
#include "../config/config"
#include "../balancer/balancer"
#include "../error/error"
#include "../stopthread/stopthread"

using namespace std;

Config config;
Balancer balancer;

static void sigcatcher (int sig) {
    if (sig == SIGHUP)
	balancer.report(true);
    else if (sig != SIGPIPE)
	balancer.terminate(true);
}

static void out_of_memory() {
    cerr << "ERROR: Out of memory\n";
    exit (1);
}

int main (int argc, char **argv) {
    static int relevant_sig[] = {
	SIGHUP, SIGINT, SIGQUIT, SIGABRT, SIGTERM, SIGSTOP, SIGPIPE
    };
    
    try {
	// Load configuration from the commandline, promote verbosity
	config.parsecmdline (argc, argv);

	ostringstream o;
	o << getpid();
	msg ("XR running as PID " + o.str() + "\n");
	
	// Load the signal handler.
	for (unsigned i = 0; i < sizeof(relevant_sig); i++)
	    signal (relevant_sig[i], sigcatcher);

	// Configure the balancer and start serving.
	balancer.init();
	balancer.serve();
    } catch (Error e) {
	cerr << e.txt() << endl;
	return (1);
    } catch (...) {
	cerr << "ERROR: Unidentified exception caught" << endl;
    }
    
    return (0);
}
