#include "sys"
#include "../error/error"

int serversocket (string addr, int port, string desc) {
    int sock;

    // Create the server socket, set options
    if ( (sock = socket (PF_INET, SOCK_STREAM, 0)) < 0 )
	throw ((Error) "Failed to create " + desc + " socket: " +
	       strerror(errno));    
    int val = 1;
    if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)))
	throw ((Error) "Failed to set socket options for " + desc + ": " +
	       strerror(errno));

    // Prepare binding
    struct sockaddr_in saddr;
    memset (&saddr, 0, sizeof(saddr));
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(port);

    // Assign interface to listen to
    ostringstream display_ip;
    if (addr[0] != '0') {
	msg ("Binding balancer to specific IP address " + addr + "\n");
	if ( (saddr.sin_addr.s_addr = inet_addr (addr.c_str())) == INADDR_NONE )
	    throw ((Error) "Cannot convert " + desc + " IP '" + addr +
		   "' to network bytes");
	display_ip << addr;
    } else {
	saddr.sin_addr.s_addr = htonl (INADDR_ANY);
	display_ip << "0.0.0.0";
    }
    display_ip << ":" << port;
    
    // Bind and listen
    if (bind (sock, (sockaddr*) &saddr, sizeof(saddr)) < 0)
	throw ((Error) "Failed to bind " + desc + " to " + display_ip.str() +
	       ": " + strerror(errno));
    if (listen (sock, 5) < 0)
	throw ((Error) "Failed to listen to " + desc + " IP " +
	       display_ip.str() + ": " + strerror(errno));

    msg ("TCP server for " + desc + " listening to " + display_ip.str() + "\n");

    return (sock);
}
    
